@@ -24,6 +24,8 @@
 #include "gail.h"
 #include "gailfactory.h"
 
+#define GNOME_ACCESSIBILITY_ENV "GNOME_ACCESSIBILITY"
+
 static gboolean gail_focus_watcher      (GSignalInvocationHint *ihint,
                                          guint                  n_param_values,
                                          const GValue          *param_values,
@@ -101,7 +103,6 @@ GAIL_ACCESSIBLE_FACTORY (GAIL_TYPE_LIST, gail_list, gail_list_new)
 GAIL_ACCESSIBLE_FACTORY (GAIL_TYPE_PANED, gail_paned, gail_paned_new)
 GAIL_ACCESSIBLE_FACTORY (GAIL_TYPE_SCROLLBAR, gail_scrollbar, gail_scrollbar_new)
 GAIL_ACCESSIBLE_FACTORY (GAIL_TYPE_OPTION_MENU, gail_option_menu, gail_option_menu_new)
-GAIL_ACCESSIBLE_FACTORY (GAIL_TYPE_CANVAS, gail_canvas, gail_canvas_new)
 GAIL_ACCESSIBLE_FACTORY (GAIL_TYPE_CHECK_MENU_ITEM, gail_check_menu_item, gail_check_menu_item_new)
 GAIL_ACCESSIBLE_FACTORY (GAIL_TYPE_RADIO_MENU_ITEM, gail_radio_menu_item, gail_radio_menu_item_new)
 GAIL_ACCESSIBLE_FACTORY (GAIL_TYPE_EXPANDER, gail_expander, gail_expander_new)
@@ -111,6 +112,9 @@ gail_get_accessible_for_widget (GtkWidget *widget,
                                 gboolean  *transient)
 {
   AtkObject *obj = NULL;
+  GType gnome_canvas;
+
+  gnome_canvas = g_type_from_name ("GnomeCanvas");
 
   *transient = FALSE;
   if (!widget)
@@ -145,17 +149,17 @@ gail_get_accessible_for_widget (GtkWidget *widget,
           g_object_unref (obj);
         }
     }
-  else if (GNOME_IS_CANVAS (widget)) 
+  else if (GTK_CHECK_TYPE ((widget), gnome_canvas))
     {
-      GnomeCanvas *canvas;
+      GObject *focused_item;
 
-      canvas = GNOME_CANVAS (widget);
+      focused_item = g_object_get_data (G_OBJECT (widget), "focused_item");
 
-      if (canvas->focused_item)
+      if (focused_item)
         {
           AtkObject *tmp;
 
-          obj = atk_gobject_accessible_for_object (G_OBJECT (canvas->focused_item));
+          obj = atk_gobject_accessible_for_object (G_OBJECT (focused_item));
           tmp = g_object_get_qdata (G_OBJECT (obj), quark_focus_object);
           if (tmp != NULL)
             obj = tmp;
@@ -867,7 +871,8 @@ gail_accessibility_module_init (void)
   gail_initialized = TRUE;
   quark_focus_object = g_quark_from_static_string ("gail-focus-object");
 
-  fprintf (stderr, "GTK Accessibility Module initialized\n");
+  if (g_getenv (GNOME_ACCESSIBILITY_ENV))
+    fprintf (stderr, "GTK Accessibility Module initialized\n");
 
   GAIL_WIDGET_SET_FACTORY (GTK_TYPE_WIDGET, gail_widget);
   GAIL_WIDGET_SET_FACTORY (GTK_TYPE_CONTAINER, gail_container);
@@ -913,12 +918,6 @@ gail_accessibility_module_init (void)
   GAIL_WIDGET_SET_FACTORY (GTK_TYPE_EXPANDER, gail_expander);
 
   /* LIBGNOMECANVAS SUPPORT */
-  GAIL_WIDGET_SET_FACTORY (GNOME_TYPE_CANVAS, gail_canvas);
-  GAIL_WIDGET_SET_FACTORY (GNOME_TYPE_CANVAS_GROUP, gail_canvas_group);
-  GAIL_WIDGET_SET_FACTORY (GNOME_TYPE_CANVAS_TEXT, gail_canvas_text);
-  GAIL_WIDGET_SET_FACTORY (GNOME_TYPE_CANVAS_RICH_TEXT, gail_canvas_text);
-  GAIL_WIDGET_SET_FACTORY (GNOME_TYPE_CANVAS_WIDGET, gail_canvas_widget);
-  GAIL_WIDGET_SET_FACTORY (GNOME_TYPE_CANVAS_ITEM, gail_canvas_item);
   GAIL_WIDGET_SET_FACTORY (GTK_TYPE_OBJECT, gail_object);
 
   atk_focus_tracker_init (gail_focus_tracker_init);
